unit AIRegistry;

interface
uses
  Windows, Classes, Registry,

  ARUtils, AInterpreter, AIHandler;

type

  TAIRegistry = class(TPersistent)
  private
    reg   :TRegistry;
  protected
    function  OpenKey(const Key:string; CanCreate:boolean):boolean;
    function  DeleteKey(const Key:string):boolean;
    procedure WriteString(const Name:string; Value:string);
    function  ReadString(const Name:string):string;

    procedure SetRootKey(const Key:HKEY);
    function  GetRootKey:HKEY;
  public
    property        RootKey:HKEY read GetRootKey write SetRootKey;

    constructor     Create;virtual;
    destructor      Destroy;override;
  end;


  TAIRegistryHandler = class(TAIHandler)
  protected
    class procedure CM_OpenKey    (obj:TObject; PropList:TAIStringList; Ex:TContext);
    class procedure CM_DeleteKey  (obj:TObject; PropList:TAIStringList; Ex:TContext);
    class procedure CM_WriteString(obj:TObject; PropList:TAIStringList; Ex:TContext);
    class procedure CM_ReadString (obj:TObject; PropList:TAIStringList; Ex:TContext);

    class procedure SetRootKey    (obj:TObject; PropList:TAIStringList; D:TData);
    class function  GetRootKey    (obj:TObject; PropList:TAIStringList):variant;

  public
    constructor     Create;override;
  end;

implementation

{ TAIRegistry }

function TAIRegistry.DeleteKey(const Key: string): boolean;
begin
  if not Assigned(reg) then reg:=TRegistry.Create();
  Result:=reg.DeleteKey(Key);
end;

function TAIRegistry.GetRootKey: HKEY;
begin
  if not Assigned(reg) then reg:=TRegistry.Create();
  Result:=reg.RootKey;
end;

function TAIRegistry.OpenKey(const Key: string; CanCreate: boolean): boolean;
begin
  if not Assigned(reg) then reg:=TRegistry.Create();
  Result:=reg.OpenKey(Key, CanCreate);
end;

function TAIRegistry.ReadString(const Name: string): string;
begin
  if not Assigned(reg) then reg:=TRegistry.Create();
  Result:=reg.ReadString(Name);
end;

procedure TAIRegistry.SetRootKey(const Key: HKEY);
begin
  if not Assigned(reg) then reg:=TRegistry.Create();
  reg.RootKey:=Key;
end;

procedure TAIRegistry.WriteString(const Name: string; Value: string);
begin
  if not Assigned(reg) then reg:=TRegistry.Create();
  reg.WriteString(Name, Value);
end;

constructor TAIRegistry.Create;
begin
  reg:=TRegistry.Create();
end;

destructor  TAIRegistry.Destroy;
begin
  if Assigned(reg) then reg.Free();

  inherited;
end;

{ TAIRegistryHandler }

class procedure TAIRegistryHandler.CM_DeleteKey(obj: TObject; PropList: TAIStringList; Ex: TContext);
var s:TData;
begin
  s:=Ex.PopDS();
  try
    Ex.PushDS(TVariant.Create(TAIRegistry(obj).DeleteKey(s.Value)));
  finally
    s.Free();
  end;
end;

class procedure TAIRegistryHandler.CM_OpenKey(obj: TObject; PropList: TAIStringList; Ex: TContext);
var s:TData;
    CanCreate:TData;
begin
  CanCreate:=Ex.PopDS();
  s:=Ex.PopDS();
  try
    Ex.PushDS(TVariant.Create(TAIRegistry(obj).OpenKey(s.Value, CanCreate.Value)));
  finally
    s.Free();
    CanCreate.Free();
  end;
end;

class procedure TAIRegistryHandler.CM_ReadString(obj: TObject; PropList: TAIStringList; Ex: TContext);
var s:TData;
begin
  s:=Ex.PopDS();
  try
    Ex.PushDS(TVariant.Create(TAIRegistry(obj).ReadString(s.Value)));
  finally
    s.Free();
  end;
end;

class procedure TAIRegistryHandler.CM_WriteString(obj: TObject; PropList: TAIStringList; Ex: TContext);
var s:TData;
    Value:TData;
begin
  Value:=Ex.PopDS();
  s:=Ex.PopDS();
  try
    TAIRegistry(obj).WriteString(s.Value, Value.Value);
  finally
    s.Free();
    Value.Free();
  end;
end;

class function TAIRegistryHandler.GetRootKey(obj: TObject; PropList: TAIStringList): variant;
begin
  case TAIRegistry(obj).RootKey of
    HKEY_CLASSES_ROOT:     Result:= 'HKEY_CLASSES_ROOT';
    HKEY_CURRENT_USER:     Result:= 'HKEY_CURRENT_USER';
    HKEY_LOCAL_MACHINE:    Result:= 'HKEY_LOCAL_MACHINE';
    HKEY_USERS:            Result:= 'HKEY_USERS';
    HKEY_PERFORMANCE_DATA: Result:= 'HKEY_PERFORMANCE_DATA';
    HKEY_CURRENT_CONFIG:   Result:= 'HKEY_CURRENT_CONFIG';
    HKEY_DYN_DATA:         Result:= 'HKEY_DYN_DATA';
  else
    Result:='';
  end;

end;

class procedure TAIRegistryHandler.SetRootKey(obj: TObject; PropList: TAIStringList; D: TData);
var s:string;
begin
  s:=D.Value;
  if CT(s, 'HKEY_CLASSES_ROOT')     then TAIRegistry(obj).RootKey:=HKEY_CLASSES_ROOT     else
  if CT(s, 'HKEY_CURRENT_USER')     then TAIRegistry(obj).RootKey:=HKEY_CURRENT_USER     else
  if CT(s, 'HKEY_LOCAL_MACHINE')    then TAIRegistry(obj).RootKey:=HKEY_LOCAL_MACHINE    else
  if CT(s, 'HKEY_USERS')            then TAIRegistry(obj).RootKey:=HKEY_USERS            else
  if CT(s, 'HKEY_PERFORMANCE_DATA') then TAIRegistry(obj).RootKey:=HKEY_PERFORMANCE_DATA else
  if CT(s, 'HKEY_CURRENT_CONFIG')   then TAIRegistry(obj).RootKey:=HKEY_CURRENT_CONFIG   else
  if CT(s, 'HKEY_DYN_DATA')         then TAIRegistry(obj).RootKey:=HKEY_DYN_DATA         else
  ;
end;

constructor TAIRegistryHandler.Create;
begin
  inherited;

  AddMethod ('OpenKey',     CM_OpenKey);
  AddMethod ('DeleteKey',   CM_DeleteKey);
  AddMethod ('WriteString', CM_WriteString);
  AddMethod ('ReadString',  CM_ReadString);

  AddProp   ('RootKey',     GetRootKey,   SetRootKey,   1);
end;

initialization
  RegisterClass   (TAIRegistry);
  RegisterHandler (TAIRegistry,  TAIRegistryHandler.Create());
end.


